Info<< "Reading thermophysical properties\n" << endl;

autoPtr<fluidThermo> pThermo
(
    fluidThermo::New(mesh)
);
fluidThermo& thermo = pThermo();
thermo.validate(args.executable(), "e");

mesh.setFluxRequired("e");

volScalarField& p = thermo.p();

volScalarField rho
(
    IOobject
    (
        "rho",
        runTime.timeName(),
        mesh,
        IOobject::READ_IF_PRESENT,
        IOobject::AUTO_WRITE
    ),
    thermo.rho()
);

// Convective velocity
Info<< "Reading field U\n" << endl;
volVectorField Uc
(
    IOobject
    (
        "Uc",
        runTime.timeName(),
        mesh,
        IOobject::MUST_READ,
        IOobject::AUTO_WRITE
    ),
    mesh
);

Info<< "Reading/calculating face flux field phi\n" << endl;

surfaceScalarField phi
(
    IOobject
    (
        "phi",
        runTime.timeName(),
        mesh,
        IOobject::READ_IF_PRESENT,
        IOobject::AUTO_WRITE
    ),
    linearInterpolate(rho*Uc) & mesh.Sf()
);

pressureControl pressureControl(p, rho, simple.dict());

mesh.setFluxRequired(p.name());

Info<< "Creating turbulence model\n" << endl;
autoPtr<compressible::turbulenceModel> turbulence
(
    compressible::turbulenceModel::New
    (
        rho,
        Uc,
        phi,
        thermo
    )
);

dimensionedScalar initialMass = fvc::domainIntegrate(rho);

#include "createMRF.H"
#include "createFvOptions.H"

bool isothermal = simple.dict().getOrDefault("isothermal", false);

// Soret's diffusion term
volVectorField mdT
(
    IOobject
    (
        "mdT",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::AUTO_WRITE
    ),
    mesh,
    dimensionedVector("mdT", dimensionSet(1,-2,-1,0,0,0,0), Foam::vector(0,0,0))
); 

if(!isothermal)
{
    mdT = thermo.mu() * 1/(2*thermo.T()) * fvc::grad(thermo.T());   
    #include "constrainMdT.H"
}


// Fick's diffusion term
volVectorField mdp
(
    IOobject
    (
        "mdp",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::AUTO_WRITE
    ),
    -thermo.mu() * (1/p * fvc::grad(p))
);


// Entire self-diffusion
volVectorField md
(
    IOobject
    (
        "md",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::AUTO_WRITE
    ),   
    mdT + mdp
);


// Diffusive velocity
volVectorField Ud
(
    IOobject
    (
        "Ud",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::AUTO_WRITE
    ),
    md/rho
);

// Total velocity 
volVectorField U
(
    IOobject
    (
        "U",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::AUTO_WRITE
    ),
    Uc + Ud
);

// Diffusive face-flux fields
surfaceScalarField phiMdT = fvc::flux(mdT);
surfaceScalarField phiMdp = fvc::flux(mdp);
surfaceScalarField phiMd = phiMdT + phiMdp;

volScalarField massRes
(
    IOobject
    (
        "massRes",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::AUTO_WRITE
    ),
    fvc::div(phi + phiMd)
);
